package pop;

import java.awt.*;
import java.awt.event.*;

public class grid extends Canvas{

public static final Color[] colors={Color.green,Color.red,Color.yellow,Color.blue,Color.orange,Color.magenta,Color.cyan};
public boolean normalizeX=false; //set in init; if true, points are started at 0 in
                                 //x direction, makes sense for "moving graphs" that
                                 //show e.g. the last ten time steps
public boolean reload=false;     //erase screen at next repaint, may be set to true
                                 //by calling app after creating new graph
private static final boolean debug=false;
private static final double granularityX=11.0;
private static final double granularityY=11.0;
private double maxX;
private double minX;
private double maxY;
private double minY;
private double maxXabs; //absolute values of the above
private double minXabs;
private double maxYabs;
private double minYabs;
private double absX;
private double absY;
private double zeroRatioX;
private double zeroRatioY;
private int widthX;
private int heightY;
public graph[] graphs;

  public grid(graph[] graphs){
    this(graphs,-5.0,10.0,-0.1,0.1);
  }

  public grid(graph[] graphs,double minX,double maxX,double minY,double maxY){
    this.graphs=graphs;
    this.minX=minX;
    this.maxX=maxX; 
    this.maxY=maxY;
    this.minY=minY;
    
    calcValues();
    setBackground(Color.white);
  }

  public void calcValues(){
    minXabs=Math.abs(minX); //absolute values of the above
    maxXabs=Math.abs(maxX); 
    maxYabs=Math.abs(maxY);
    minYabs=Math.abs(minY);

    absX=Math.abs(maxX)+Math.abs(minX); //total interval length
    absY=Math.abs(maxY)+Math.abs(minY);

    zeroRatioX=minXabs/(maxXabs+minXabs);
    zeroRatioY=minYabs/(maxYabs+minYabs);
}

    //convert mathematical X coordinate in screen coordinate
public int transX(double x){
  int returnX=(int) ((zeroRatioX+(x/absX))*widthX);
  if(debug) System.out.println("X: "+x+" -> "+returnX);
  return returnX;
}

    //convert mathematical Y coordinate in screen coordinate
public int transY(double y){
  int returnY=heightY - (int) ((zeroRatioY+(y/absY))*heightY);
  if(debug) System.out.println("Y: "+y+" -> "+returnY);
  return returnY;
}

private void drawLine(double x1,double y1,double x2, double y2, Graphics g){
g.drawLine(transX(x1),transY(y1),transX(x2),transY(y2));
}

private void drawLine(Double x1,Double y1,Double x2, Double y2, Graphics g){
g.drawLine(transX(x1.doubleValue()),transY(y1.doubleValue()),transX(x2.doubleValue()),transY(y2.doubleValue()));
}

private void drawLine(Double x1,Double y1,Double x2, Double y2, Double normX, Graphics g){
g.drawLine(transX(x1.doubleValue()-normX.doubleValue()),transY(y1.doubleValue()),transX(x2.doubleValue()-normX.doubleValue()),transY(y2.doubleValue()));
}

private void drawPoint(Double x1,Double y1, Double normX, Graphics g){
g.drawRect(transX(x1.doubleValue()-normX.doubleValue()),transY(y1.doubleValue()),1,1);
}

//call with lower left corner and upper right corner coordinates
private void drawRect(Double x1,Double y1, Double x2,Double y2, Double normX, Graphics g){
g.drawRect(transX(x1.doubleValue()-normX.doubleValue()),transY(y1.doubleValue()),transX(x2.doubleValue())-transX(x1.doubleValue()+normX.doubleValue()),transY(y1.doubleValue())-transY(y2.doubleValue()));
/*
System.out.println("x>"+(x1.doubleValue()-normX.doubleValue()));
System.out.println("y>"+(y1.doubleValue()));
System.out.println("width>"+(transX(x2.doubleValue())-transX(x1.doubleValue()+normX.doubleValue())));
System.out.println("height>"+(transY(y2.doubleValue())-transY(y1.doubleValue())));
System.out.println("y2:"+y2.doubleValue());
System.out.println("trans y2:"+transY(y2.doubleValue()));
System.out.println("y1:"+y1.doubleValue());
System.out.println("trans y1:"+transY(y1.doubleValue()));
*/
}

public void drawGraph(Object[] xs, Object[] ys, int drawMode, Graphics g){
Double normX;
if(normalizeX)
  normX=(Double) xs[0];
else
  normX=new Double(0.0);
for(int n=0;n<xs.length;n++){
  try{
  switch(drawMode){
      case 0: drawLine((Double) xs[n],(Double) ys[n],(Double) xs[n+1],(Double) ys[n+1],normX,g); break;
      case 1: drawPoint((Double) xs[n],(Double) ys[n],normX,g); break;
      case 2: drawRect((Double) xs[n],new Double(0.0),(Double) xs[n+1],(Double) ys[n],normX,g); break;
      }
    }catch(ArrayIndexOutOfBoundsException x){}//do nothing
  }
}

public void drawGraphs(Graphics g){
if(graphs!=null)
  for(int i=0;i<graphs.length;i++){
    g.setColor(colors[i]);
    drawGraph(graphs[i].getGraphXs(),graphs[i].getGraphYs(), graphs[i].drawMode,g);
    }
}

public void drawString(String what,double xi,double yi,Graphics g){
int x=transX(xi);
int y=transY(yi);
g.drawString(what,x,y);
}

public String shorten(String in){
if(in.length()>5)       //exactness of the numbers on the grid
  in=in.substring(0,4);           // - to long numbers disturb
while(in.endsWith("0") && in.indexOf(".")!=-1)
  in=in.substring(0,in.length()-1);//kill zeros after the dot
return in;
}

public void update(Graphics g){
  paint(g);
}

public void paint(Graphics g){
  Dimension size=getSize();
  widthX=getWidth();
  heightY=getHeight();
  if(normalizeX||reload)            //we have to repaint the graph
    g.clearRect(0,0,widthX,heightY);//since it may move
  reload=false;
//kleinen font einstellen
  Font temp=g.getFont();
  g.setFont(temp.deriveFont((float) 10.0));

  g.setColor(Color.lightGray);
  for(int n=0;n<granularityX;n++){
    double xVal=minX+n*(absX/granularityX);
    drawLine(xVal,minY,xVal,maxY,g);
    drawString(shorten(""+xVal),xVal+(absX/300),0.0,g);
    }
  for(int n=0;n<granularityY;n++){
    double yVal=minY+n*(absY/granularityY);
    drawLine(minX,yVal,maxX,yVal,g);
    drawString(shorten(""+yVal),0.0+(absX/300),yVal,g);
    }
  g.setColor(Color.black);

//x- und y-Achse malen
  drawLine(minX,0.0,maxX,0.0,g);
  drawLine(0.0,minY,0.0,maxY,g);

  drawGraphs(g);
  }
}


